@@ -30,7 +30,7 @@
 
 /**
  * SECTION:stylescheme
- * @Short_description: Object controlling the appearance of GtkSourceView
+ * @Short_description: Controls the appearance of GtkSourceView
  * @Title: GtkSourceStyleScheme
  * @See_also: #GtkSourceStyle, #GtkSourceStyleSchemeManager
  *
@@ -40,8 +40,7 @@
  * non-highlighted text, color for the line numbers, etc.
  *
  * Style schemes are stored in XML files. The format of a scheme file is
- * documented in the
- * <link linkend="style-reference">style scheme reference</link>.
+ * documented in the [style scheme reference][style-reference].
  */
 
 #define STYLE_TEXT			"text"
@@ -53,8 +52,10 @@
 #define STYLE_SECONDARY_CURSOR		"secondary-cursor"
 #define STYLE_CURRENT_LINE		"current-line"
 #define STYLE_LINE_NUMBERS		"line-numbers"
+#define STYLE_CURRENT_LINE_NUMBER	"current-line-number"
 #define STYLE_RIGHT_MARGIN		"right-margin"
 #define STYLE_DRAW_SPACES		"draw-spaces"
+#define STYLE_BACKGROUND_PATTERN	"background-pattern"
 
 #define STYLE_SCHEME_VERSION		"1.0"
 
@@ -82,7 +83,8 @@ struct _GtkSourceStyleSchemePrivate
 	GHashTable *style_cache;
 	GHashTable *named_colors;
 
-	GtkCssProvider *css;
+	GtkCssProvider *css_provider;
+	GtkCssProvider *css_provider_cursors;
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (GtkSourceStyleScheme, gtk_source_style_scheme, G_TYPE_OBJECT)
@@ -111,7 +113,8 @@ gtk_source_style_scheme_dispose (GObject *object)
 	}
 
 	g_clear_object (&scheme->priv->parent);
-	g_clear_object (&scheme->priv->css);
+	g_clear_object (&scheme->priv->css_provider);
+	g_clear_object (&scheme->priv->css_provider_cursors);
 
 	G_OBJECT_CLASS (gtk_source_style_scheme_parent_class)->dispose (object);
 }
@@ -208,8 +211,8 @@ gtk_source_style_scheme_class_init (GtkSourceStyleSchemeClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_ID,
 					 g_param_spec_string ("id",
-						 	      _("Style scheme id"),
-							      _("Style scheme id"),
+						 	      "Style scheme id",
+							      "Style scheme id",
 							      NULL,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
@@ -221,8 +224,8 @@ gtk_source_style_scheme_class_init (GtkSourceStyleSchemeClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_NAME,
 					 g_param_spec_string ("name",
-						 	      _("Style scheme name"),
-							      _("Style scheme name"),
+						 	      "Style scheme name",
+							      "Style scheme name",
 							      NULL,
 							      G_PARAM_READABLE));
 
@@ -234,8 +237,8 @@ gtk_source_style_scheme_class_init (GtkSourceStyleSchemeClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_DESCRIPTION,
 					 g_param_spec_string ("description",
-						 	      _("Style scheme description"),
-							      _("Style scheme description"),
+						 	      "Style scheme description",
+							      "Style scheme description",
 							      NULL,
 							      G_PARAM_READABLE));
 
@@ -247,8 +250,8 @@ gtk_source_style_scheme_class_init (GtkSourceStyleSchemeClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_FILENAME,
 					 g_param_spec_string ("filename",
-						 	      _("Style scheme filename"),
-							      _("Style scheme filename"),
+						 	      "Style scheme filename",
+							      "Style scheme filename",
 							      NULL,
 							      G_PARAM_READABLE));
 }
@@ -276,7 +279,7 @@ gtk_source_style_scheme_init (GtkSourceStyleScheme *scheme)
 	scheme->priv->named_colors = g_hash_table_new_full (g_str_hash, g_str_equal,
 							    g_free, g_free);
 
-	scheme->priv->css = gtk_css_provider_new ();
+	scheme->priv->css_provider = gtk_css_provider_new ();
 }
 
 /**
@@ -369,32 +372,6 @@ gtk_source_style_scheme_get_filename (GtkSourceStyleScheme *scheme)
 	return scheme->priv->filename;
 }
 
-/**
- * _gtk_source_style_scheme_new:
- * @id: scheme id.
- * @name: scheme name.
- *
- * Returns: new empty #GtkSourceStyleScheme.
- *
- * Since: 2.0
- */
-GtkSourceStyleScheme *
-_gtk_source_style_scheme_new (const gchar *id,
-			      const gchar *name)
-{
-	GtkSourceStyleScheme *scheme;
-
-	g_return_val_if_fail (id != NULL, NULL);
-	g_return_val_if_fail (name != NULL, NULL);
-
-	scheme = g_object_new (GTK_SOURCE_TYPE_STYLE_SCHEME,
-			       "id", id,
-			       "name", name,
-			       NULL);
-
-	return scheme;
-}
-
 /*
  * get_color_by_name:
  * @scheme: a #GtkSourceStyleScheme.
@@ -635,50 +612,113 @@ _gtk_source_style_scheme_get_current_line_color (GtkSourceStyleScheme *scheme,
 	return get_color (style, FALSE, color);
 }
 
-static void
-update_cursor_colors (GtkWidget      *widget,
-		      GtkSourceStyle *style_primary,
-		      GtkSourceStyle *style_secondary)
+/*
+ * Returns TRUE if the style for background-pattern-color is set in the scheme
+ */
+gboolean
+_gtk_source_style_scheme_get_background_pattern_color (GtkSourceStyleScheme *scheme,
+                                                       GdkRGBA              *color)
 {
-	GdkRGBA primary_color;
-	GdkRGBA secondary_color;
-	GdkRGBA *primary = NULL;
-	GdkRGBA *secondary = NULL;
+	GtkSourceStyle *style;
 
-	if (get_color (style_primary, TRUE, &primary_color))
+	g_return_val_if_fail (GTK_SOURCE_IS_STYLE_SCHEME (scheme), FALSE);
+	g_return_val_if_fail (color != NULL, FALSE);
+
+	style = gtk_source_style_scheme_get_style (scheme, STYLE_BACKGROUND_PATTERN);
+
+	return get_color (style, FALSE, color);
+}
+
+static gchar *
+get_cursors_css_style (GtkSourceStyleScheme *scheme,
+		       GtkWidget            *widget)
+{
+	GtkSourceStyle *primary_style;
+	GtkSourceStyle *secondary_style;
+	GdkRGBA primary_color = { 0 };
+	GdkRGBA secondary_color = { 0 };
+	gchar *primary_color_str;
+	gchar *secondary_color_str;
+	gchar *css;
+
+	primary_style = gtk_source_style_scheme_get_style (scheme, STYLE_CURSOR);
+	secondary_style = gtk_source_style_scheme_get_style (scheme, STYLE_SECONDARY_CURSOR);
+
+	if (!get_color (primary_style, TRUE, &primary_color))
 	{
-		primary = &primary_color;
+		return NULL;
 	}
 
-	if (get_color (style_secondary, TRUE, &secondary_color))
-	{
-		secondary = &secondary_color;
-	}
-
-	if (primary != NULL && secondary == NULL)
+	if (!get_color (secondary_style, TRUE, &secondary_color))
 	{
 		GtkStyleContext *context;
 
 		context = gtk_widget_get_style_context (widget);
-		gtk_style_context_get_background_color (context, GTK_STATE_FLAG_NORMAL,
-		                                        &secondary_color);
+
+		gtk_style_context_get (context,
+				       GTK_STATE_FLAG_NORMAL,
+				       "background-color", &secondary_color,
+				       NULL);
 
 		/* shade the secondary cursor */
 		secondary_color.red *= 0.5;
 		secondary_color.green *= 0.5;
 		secondary_color.blue *= 0.5;
-
-		secondary = &secondary_color;
 	}
 
-	if (primary != NULL)
+	primary_color_str = gdk_rgba_to_string (&primary_color);
+	secondary_color_str = gdk_rgba_to_string (&secondary_color);
+
+	css = g_strdup_printf ("GtkSourceView {\n"
+			       "\t-GtkWidget-cursor-color: %s;\n"
+			       "\t-GtkWidget-secondary-cursor-color: %s;\n"
+			       "}\n",
+			       primary_color_str,
+			       secondary_color_str);
+
+	g_free (primary_color_str);
+	g_free (secondary_color_str);
+
+	return css;
+}
+
+/* The CssProvider for the cursors depends only on @scheme, but it needs a
+ * @widget to shade the background color in case the secondary cursor color
+ * isn't defined. The background color is normally defined by @scheme, or if
+ * it's not defined it is taken from the GTK+ theme. So ideally, if the GTK+
+ * theme changes at runtime, we should regenerate the CssProvider for the
+ * cursors, but it isn't done.
+ */
+static GtkCssProvider *
+get_css_provider_cursors (GtkSourceStyleScheme *scheme,
+			  GtkWidget            *widget)
+{
+	gchar *css;
+	GtkCssProvider *provider;
+	GError *error = NULL;
+
+	css = get_cursors_css_style (scheme, widget);
+
+	if (css == NULL)
 	{
-		gtk_widget_override_cursor (widget, primary, secondary);
+		return NULL;
 	}
-	else
+
+	provider = gtk_css_provider_new ();
+
+	gtk_css_provider_load_from_data (provider, css, -1, &error);
+	g_free (css);
+
+	if (error != NULL)
 	{
-		gtk_widget_override_cursor (widget, NULL, NULL);
+		g_warning ("Error when loading CSS for cursors: %s", error->message);
+		g_error_free (error);
+
+		g_object_unref (provider);
+		return NULL;
 	}
+
+	return provider;
 }
 
 /**
@@ -694,7 +734,6 @@ void
 _gtk_source_style_scheme_apply (GtkSourceStyleScheme *scheme,
 				GtkWidget            *widget)
 {
-	GtkSourceStyle *style, *style2;
 	GtkStyleContext *context;
 
 	g_return_if_fail (GTK_SOURCE_IS_STYLE_SCHEME (scheme));
@@ -703,7 +742,7 @@ _gtk_source_style_scheme_apply (GtkSourceStyleScheme *scheme,
 	/* we need to translate some of the style scheme properties in a CSS override */
 	context = gtk_widget_get_style_context (GTK_WIDGET (widget));
 	gtk_style_context_add_provider (context,
-	                                GTK_STYLE_PROVIDER (scheme->priv->css),
+	                                GTK_STYLE_PROVIDER (scheme->priv->css_provider),
 	                                GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 
 	G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
@@ -711,9 +750,24 @@ _gtk_source_style_scheme_apply (GtkSourceStyleScheme *scheme,
 	gtk_style_context_invalidate (context);
 	G_GNUC_END_IGNORE_DEPRECATIONS;
 
-	style = gtk_source_style_scheme_get_style (scheme, STYLE_CURSOR);
-	style2 = gtk_source_style_scheme_get_style (scheme, STYLE_SECONDARY_CURSOR);
-	update_cursor_colors (widget, style, style2);
+	/* The CssProvider for the cursors needs that the first provider is
+	 * applied, to get the background color.
+	 */
+	if (scheme->priv->css_provider_cursors == NULL)
+	{
+		scheme->priv->css_provider_cursors = get_css_provider_cursors (scheme, widget);
+	}
+
+	if (scheme->priv->css_provider_cursors != NULL)
+	{
+		gtk_style_context_add_provider (context,
+						GTK_STYLE_PROVIDER (scheme->priv->css_provider_cursors),
+						GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+
+		G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
+		gtk_style_context_invalidate (context);
+		G_GNUC_END_IGNORE_DEPRECATIONS;
+	}
 }
 
 /**
@@ -736,14 +790,18 @@ _gtk_source_style_scheme_unapply (GtkSourceStyleScheme *scheme,
 
 	context = gtk_widget_get_style_context (GTK_WIDGET (widget));
 	gtk_style_context_remove_provider (context,
-	                                   GTK_STYLE_PROVIDER (scheme->priv->css));
+	                                   GTK_STYLE_PROVIDER (scheme->priv->css_provider));
+
+	if (scheme->priv->css_provider_cursors != NULL)
+	{
+		gtk_style_context_remove_provider (context,
+						   GTK_STYLE_PROVIDER (scheme->priv->css_provider_cursors));
+	}
 
 	G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
 	/* See https://bugzilla.gnome.org/show_bug.cgi?id=708583 */
 	gtk_style_context_invalidate (context);
 	G_GNUC_END_IGNORE_DEPRECATIONS;
-
-	update_cursor_colors (widget, NULL, NULL);
 }
 
 /* --- PARSER ---------------------------------------------------------------- */
@@ -821,9 +879,9 @@ generate_css_style (GtkSourceStyleScheme *scheme)
 	append_css_style (final_style, style, ".view:selected:focused");
 
 	style2 = gtk_source_style_scheme_get_style (scheme, STYLE_SELECTED_UNFOCUSED);
-	if (style2 == NULL)
-		style2 = style;
-	append_css_style (final_style, style2, ".view:selected");
+	append_css_style (final_style,
+			  style2 != NULL ? style2 : style,
+			  ".view:selected");
 
 	/* For now we use "line numbers" colors for all the gutters */
 	style = gtk_source_style_scheme_get_style (scheme, STYLE_LINE_NUMBERS);
@@ -840,12 +898,20 @@ generate_css_style (GtkSourceStyleScheme *scheme)
 		append_css_style (final_style, style, "GtkSourceView");
 	}
 
+	style = gtk_source_style_scheme_get_style (scheme, STYLE_CURRENT_LINE_NUMBER);
+	if (style != NULL)
+	{
+		append_css_style (final_style, style, ".current-line-number");
+	}
+
 	if (*final_style->str != '\0')
 	{
 		GError *error = NULL;
 
-		if (!gtk_css_provider_load_from_data (scheme->priv->css, final_style->str,
-		                                      final_style->len, &error))
+		if (!gtk_css_provider_load_from_data (scheme->priv->css_provider,
+						      final_style->str,
+		                                      final_style->len,
+						      &error))
 		{
 			g_warning ("%s", error->message);
 			g_error_free (error);
